﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;

namespace HeuristicLab.Persistence {

  /// <summary>
  /// Mark a class to be considered by the <c>StorableSerializer</c>.
  /// </summary>
  [AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct | AttributeTargets.Interface | AttributeTargets.Enum | AttributeTargets.Delegate, Inherited = false, AllowMultiple = false)]
  public sealed class StorableTypeAttribute : Attribute {


    /// <summary>
    /// Specify how members are selected for serialization.
    /// </summary>
    public StorableMemberSelection MemberSelection { get; private set; }

    public Guid Guid { get; private set; }

    /// <summary>
    /// Mark a class to be serialize by the <c>StorableSerizlier</c>
    /// </summary>
    /// <param name="memberSelection">The storable class memberSelection.</param>
    public StorableTypeAttribute(StorableMemberSelection memberSelection, string guid) {
      MemberSelection = memberSelection;
      Guid = new Guid(guid);
    }

    public StorableTypeAttribute(string guid) {
      Guid = new Guid(guid);
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="StorableTypeAttribute"/> class.
    /// The default value for <see cref="StorableMemberSelection"/> is
    /// <see cref="StorableMemberSelection.MarkedOnly"/>.
    /// </summary>
    private StorableTypeAttribute() { }

    /// <summary>
    ///  Checks if the <see cref="StorableTypeAttribute"/> is present on a memberSelection.
    /// </summary>
    /// <param name="type">The memberSelection which should be checked for the <see cref="StorableTypeAttribute"/></param>
    /// <returns></returns>
    public static bool IsStorableType(Type type) {
      object[] attribs = type.GetCustomAttributes(typeof(StorableTypeAttribute), false);
      return attribs.Length > 0;
    }

    public static StorableTypeAttribute GetStorableTypeAttribute(Type type) {
      return (StorableTypeAttribute)Attribute.GetCustomAttribute(type, typeof(StorableTypeAttribute), false);
    }

  }
}

